@@ -137,6 +137,81 @@ def ncx2pdf(x, k, l):
 	return numpy.exp(ncx2logpdf(x, k, l))
 
 
+#
+# =============================================================================
+#
+#                                Poisson Stuff
+#
+# =============================================================================
+#
+
+
+def assert_probability(f):
+	def g(*args, **kwargs):
+		p = f(*args, **kwargs)
+		if isinstance(p, numpy.ndarray):
+			assert ((0. <= p) & (p <= 1.)).all()
+		else:
+			assert 0. <= p <= 1.
+		return p
+	return g
+
+
+@assert_probability
+@numpy.vectorize
+def poisson_p_not_0(l):
+	"""
+	Return the probability that a Poisson process with a mean rate of l
+	yields a non-zero count.  = 1 - exp(-l).
+	"""
+	assert l >= 0.
+
+	# need -l everywhere
+
+	l = -l
+
+	#
+	# result is closer to 1 than to 0.  use direct evaluation.
+	#
+
+	if l < -0.69314718055994529:
+		return 1. - math.exp(l)
+
+	#
+	# result is closer to 0 than to 1.  use Taylor expansion for exp()
+	# with leading term removed to avoid having to subtract from 1 to
+	# get answer.
+	#
+	# 1 - exp x = -(x + x^2/2! + x^3/3! + ...)
+	#
+
+	s = [l]
+	term = l
+	threshold = -1e-20 * l
+	for n in itertools.count(2):
+		term *= l / n
+		s.append(term)
+		if abs(term) <= threshold:
+			# smallest term was added last, want to compute sum
+			# from smallest to largest
+			s.reverse()
+			s = -sum(s)
+			# if the sum was identically 0 then we've ended up
+			# with -0.0 which we add a special case for to make
+			# positive.
+			assert s >= 0.
+			return s if s else 0.
+
+
+@assert_probability
+def poisson_p_0(l):
+	"""
+	Return the probability that a Poisson process with a mean rate of l
+	yields a zero count.  = exp(-l).
+	"""
+	return numpy.exp(-l)
+
+
 #
 # =============================================================================
 #
@@ -146,6 +221,7 @@ def ncx2pdf(x, k, l):
 #
 
 
+@assert_probability
 def fap_after_trials(p, m):
 	"""
 	Given the probability, p, that an event occurs, compute the
@@ -281,7 +357,9 @@ def fap_after_trials(p, m):
 				return sum(s)
 
 	#
-	# compute result as 1 - exp(m * log(1 - p))
+	# compute result as 1 - exp(m * log(1 - p)).  use poisson_p_not_0()
+	# to evaluate 1 - exp() with an algorithm that avoids loss of
+	# precision.
 	#
 
 	try:
@@ -297,29 +375,10 @@ def fap_after_trials(p, m):
 
 		return 1.
 
-	if x > -0.69314718055994529:
-		#
-		# result is closer to 0 than to 1.  use Taylor expansion
-		# for exp() with leading term removed to avoid having to
-		# subtract from 1 to get answer.
-		#
-		# 1 - exp x = -(x + x^2/2! + x^3/3! + ...)
-		#
-
-		s = [x]
-		term = x
-		for n in itertools.count(2):
-			term *= x / n
-			s.append(term)
-			# 0th term is always negative
-			if abs(term) <= -1e-18 * s[0]:
-				s.reverse()
-				return -sum(s)
-
-	return 1. - math.exp(x)
+	return float(poisson_p_not_0(-x))
 
 
-fap_after_trials_arr = numpy.frompyfunc(fap_after_trials, 2, 1)
+fap_after_trials_arr = numpy.vectorize(fap_after_trials)
 
 
 def trials_from_faps(p0, p1):
@@ -683,6 +742,14 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 	# FIXME:  must ensure lower boundary matches search threshold
 	snr_min = 4.
 
+	# with what weight to include the denominator PDFs in the
+	# numerator.  numerator will be
+	#
+	# numerator =
+	#   (1 - accidental_weight) * (measured numerator) +
+	#   accidental_weight * (measured denominator)
+	numerator_accidental_weight = 0.
+
 	# if two horizon distances, D1 and D2, differ by less than
 	#
 	#	| ln(D1 / D2) | <= log_distance_tolerance
@@ -695,9 +762,13 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 	@staticmethod
 	def quantize_horizon_distances(horizon_distances, log_distance_tolerance = PosInf, min_ratio = 0.1):
 		horizon_distance_norm = max(horizon_distances.values())
-		assert horizon_distance_norm != 0.
+		assert horizon_distance_norm >= 0.
+		# check for no-op:  all distances are 0.
+		if horizon_distance_norm == 0.:
+			return dict.fromkeys(horizon_distances, 0.)
+		# check for no-op:  map all (non-zero) values to 1
 		if math.isinf(log_distance_tolerance):
-			return dict((instrument, 1.) for instrument in horizon_distances)
+			return dict((instrument, 1. if horizon_distance > 0. else 0.) for instrument, horizon_distance in horizon_distances.items())
 		min_distance = min_ratio * horizon_distance_norm
 		return dict((instrument, (0. if horizon_distance < min_distance else math.exp(round(math.log(horizon_distance / horizon_distance_norm) / log_distance_tolerance) * log_distance_tolerance))) for instrument, horizon_distance in horizon_distances.items())
 
@@ -733,6 +804,9 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 			"E0_snr_chi": self.pdf_from_rates_snrchi2,
 		}
 
+		# set to True to include zero-lag histograms in background model
+		self.zero_lag_in_background = False
+
 	def __iadd__(self, other):
 		# NOTE:  because we use custom PDF constructions, the stock
 		# .__iadd__() method for this class will not result in
@@ -866,6 +940,13 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		return params
 
 	def lnP_signal(self, params):
+		# NOTE:  lnP_signal() and lnP_noise() (not shown here) both
+		# omit the factor P(horizon distance) = 1/T because it is
+		# identical in the numerator and denominator and so factors
+		# out of the ranking statistic, and because it is constant
+		# and so equivalent to an irrelevant normalization factor
+		# in the ranking statistic PDF sampler code.
+
 		# (instrument, snr) pairs sorted alphabetically by
 		# instrument name
 		snrs = sorted((name.split("_")[0], value[0]) for name, value in params.items() if name.endswith("_snr_chi"))
@@ -880,19 +961,7 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		# probabilities to is OK.  we probably need to cache these
 		# and save them in the XML file, too, like P(snrs | signal,
 		# instruments)
-		lnP_signal += super(ThincaCoincParamsDistributions, self).lnP_signal(params)
-
-		# return logarithm of (.99 P(..|signal) + 0.01 P(..|noise))
-		# FIXME:  investigate how to determine correct mixing ratio
-		lnP_noise = self.lnP_noise(params)
-		if math.isinf(lnP_noise) and math.isinf(lnP_signal):
-			if lnP_noise < 0. and lnP_signal < 0.:
-				return NegInf
-			if lnP_noise > 0. and lnP_signal > 0.:
-				return PosInf
-		lnP_signal += -0.010050335853501451	# math.log(.99)
-		lnP_noise += -4.6051701859880909	# math.log(0.01)
-		return max(lnP_signal, lnP_noise) + math.log1p(math.exp(-abs(lnP_signal - lnP_noise)))
+		return lnP_signal + super(ThincaCoincParamsDistributions, self).lnP_signal(params)
 
 	def add_snrchi_prior(self, rates_dict, n, prefactors_range, df, inv_snr_pow = 4., verbose = False):
 		if verbose:
@@ -1055,9 +1124,6 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 			self.background_rates["instruments"][self.instrument_categories.category(instruments),] = count * coincidence_bins
 
 	def add_foreground_snrchi_prior(self, n, prefactors_range = (0.01, 0.25), df = 40, inv_snr_pow = 4., verbose = False):
-		for instrument, number_of_events in n.items():
-			# NOTE a uniform prior is added that must be smaller than the uniform prior added for the background
-			self.injection_rates["%s_snr_chi" % instrument].array += 1. / number_of_events / self.injection_rates["%s_snr_chi" % instrument].array.size / 1e20
 		self.add_snrchi_prior(self.injection_rates, n, prefactors_range, df, inv_snr_pow = inv_snr_pow, verbose = verbose)
 
 	def populate_prob_of_instruments_given_signal(self, segs, n = 1., verbose = False):
@@ -1113,17 +1179,47 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 			self.zero_lag_lnpdf_interp["instruments"] = mkinterp(self.zero_lag_pdf["instruments"])
 
 	def pdf_from_rates_instruments(self, key, pdf_dict):
-		# instrument combos are probabilities, not densities.  be
-		# sure the single-instrument categories are zeroed.
+		# get the binned array we're going to process
 		binnedarray = pdf_dict[key]
+
+		# optionally include zero-lag instrument combo counts in
+		# the background counts
+		if self.zero_lag_in_background and pdf_dict is self.background_pdf:
+			binnedarray.array += self.zero_lag_rates[key].array
+
+		# be sure the single-instrument categories are zeroed.
 		for category in self.instrument_categories.values():
 			binnedarray[category,] = 0
+
+		# optionally mix denominator into numerator
+		if pdf_dict is self.injection_pdf and self.numerator_accidental_weight:
+			denom = self.background_rates[key].array.copy()
+			if self.zero_lag_in_background:
+				denom += self.zero_lag_rates[key].array
+			for category in self.instrument_categories.values():
+				denom[category,] = 0
+			binnedarray.array += denom * (binnedarray.array.sum() / denom.sum() * self.numerator_accidental_weight)
+
+		# instrument combos are probabilities, not densities.
 		with numpy.errstate(invalid = "ignore"):
 			binnedarray.array /= binnedarray.array.sum()
 
 	def pdf_from_rates_snrchi2(self, key, pdf_dict, snr_kernel_width_at_8 = 10., chisq_kernel_width = 0.1,  sigma = 10.):
 		# get the binned array we're going to process
 		binnedarray = pdf_dict[key]
+
+		# optionally include zero-lag (SNR,\chi^2) counts in the
+		# background counts
+		if self.zero_lag_in_background and pdf_dict is self.background_pdf:
+			binnedarray.array += self.zero_lag_rates[key].array
+
+		# optionally mix denominator into numerator
+		if pdf_dict is self.injection_pdf and self.numerator_accidental_weight:
+			denom = self.background_rates[key].array.copy()
+			if self.zero_lag_in_background:
+				denom += self.zero_lag_rates[key].array
+			binnedarray.array += denom * (binnedarray.array.sum() / denom.sum() * self.numerator_accidental_weight)
+
 		numsamples = binnedarray.array.sum() / 10. + 1. # Be extremely conservative and assume only 1 in 10 samples are independent.
 		# construct the density estimation kernel
 		snr_bins = binnedarray.bins[0]
@@ -1144,11 +1240,8 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 			chisq_kernel_bins = 2.5
 			warnings.warn("Replacing chisq kernel bins with 2.5")
 
-		# Compute the KDE
-		kernel = rate.gaussian_window(snr_kernel_bins, chisq_kernel_bins, sigma = sigma)
-
-		# convolve with the bin count data
-		rate.filter_array(binnedarray.array, kernel)
+		# convolve bin count with density estimation kernel
+		rate.filter_array(binnedarray.array, rate.gaussian_window(snr_kernel_bins, chisq_kernel_bins, sigma = sigma))
 
 		# zero everything below the SNR cut-off.  need to do the
 		# slicing ourselves to avoid zeroing the at-threshold bin
@@ -1321,7 +1414,7 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		simply be the probability densities of the signal and noise
 		populations at that point in parameter space.  They cannot
 		be used to form an importance weighted sampler of the log
-		likeklihood ratios.
+		likelihood ratios.
 		"""
 		#
 		# retrieve horizon distance from history if not given
@@ -1399,7 +1492,7 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 
 
 	@classmethod
-	def joint_pdf_of_snrs(cls, instruments, inst_horiz_mapping, n_samples = 160000, bins = rate.ATanLogarithmicBins(3.6, 120., 100), progressbar = None):
+	def joint_pdf_of_snrs(cls, instruments, inst_horiz_mapping, n_samples = 160000, bins = rate.ATanLogarithmicBins(3.6, 1200., 170), progressbar = None):
 		"""
 		Return a BinnedArray representing the joint probability
 		density of measuring a set of SNRs from a network of
@@ -1427,7 +1520,7 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		# has infinite size, we remove it from the sequence
 		# (meaning the PDF will be left 0 in that bin).
 		pdf = rate.BinnedArray(rate.NDBins([bins] * len(instruments)))
-		snr_sequence = rate.ATanLogarithmicBins(3.6, 120., 300)
+		snr_sequence = rate.ATanLogarithmicBins(3.6, 1200., 500)
 		snr_snrlo_snrhi_sequence = numpy.array(zip(snr_sequence.centres(), snr_sequence.lower(), snr_sequence.upper())[:-1])
 
 		# compute the SNR at which to begin iterations over bins
@@ -1467,14 +1560,14 @@ class ThincaCoincParamsDistributions(snglcoinc.CoincParamsDistributions):
 		for i in xrange(n_samples):
 			# select random sky location and source orbital
 			# plane inclination and choice of polarization
-			theta = acos(random_uniform(-1., 1.))
-			phi = random_uniform(0., twopi)
+			ra = random_uniform(0., twopi)
+			dec = pi_2 - acos(random_uniform(-1., 1.))
 			psi = random_uniform(0., twopi)
 			cosi2 = random_uniform(-1., 1.)**2.
 
 			# F+^2 and Fx^2 for each instrument
-			fpfc2 = numpy.array(tuple(xlal_am_resp(resp, phi, pi_2 - theta, psi, gmst) for resp in resps))**2.
-			fpfc2_other = numpy.array(tuple(xlal_am_resp(resp, phi, pi_2 - theta, psi, gmst) for resp in resps_other))**2.
+			fpfc2 = numpy.array(tuple(xlal_am_resp(resp, ra, dec, psi, gmst) for resp in resps))**2.
+			fpfc2_other = numpy.array(tuple(xlal_am_resp(resp, ra, dec, psi, gmst) for resp in resps_other))**2.
 
 			# ratio of distance to inverse SNR for each instrument
 			fpfc_factors = ((1. + cosi2)**2. / 4., cosi2)
@@ -1630,7 +1723,8 @@ def P_instruments_given_signal(horizon_history, n_samples = 500000, min_distance
 	xlal_am_resp = lal.ComputeDetAMResponse
 
 	# loop as many times as requested
-	for i in xrange(n_samples):
+	successes = fails = 0
+	while successes < n_samples:
 		# retrieve random horizon distances in the same order as
 		# the instruments.  note:  rand_horizon_distances() is only
 		# evaluated once in this expression.  that's important
@@ -1638,14 +1732,14 @@ def P_instruments_given_signal(horizon_history, n_samples = 500000, min_distance
 
 		# select random sky location and source orbital plane
 		# inclination and choice of polarization
-		theta = acos(random_uniform(-1., 1.))
-		phi = random_uniform(0., twopi)
+		ra = random_uniform(0., twopi)
+		dec = pi_2 - acos(random_uniform(-1., 1.))
 		psi = random_uniform(0., twopi)
 		cosi2 = random_uniform(-1., 1.)**2.
 
 		# compute F+^2 and Fx^2 for each antenna from the sky
 		# location and antenna responses
-		fpfc2 = numpy_array(tuple(xlal_am_resp(resp, phi, pi_2 - theta, psi, gmst) for resp in resps))**2.
+		fpfc2 = numpy_array(tuple(xlal_am_resp(resp, ra, dec, psi, gmst) for resp in resps))**2.
 
 		# 1/8 ratio of inverse SNR to distance for each instrument
 		# (1/8 because horizon distance is defined for an SNR of 8,
@@ -1691,6 +1785,9 @@ def P_instruments_given_signal(horizon_history, n_samples = 500000, min_distance
 		if V[0] <= min_distance:
 			# fewer than two instruments are on, so no
 			# combination can see anything
+			fails += 1
+			if successes + fails >= n_samples and fails / float(successes + fails) > .90:
+				raise ValueError("convergence too slow:  success/fail ratio = %d/%d" % (successes, fails))
 			continue
 
 		# for each instrument combination, probability that a
@@ -1706,6 +1803,7 @@ def P_instruments_given_signal(horizon_history, n_samples = 500000, min_distance
 		# other combination of instruments.
 		for key, p, pnext in zip(instruments, P, P[1:] + (0.,)):
 			result[key] += p - pnext
+		successes += 1
 	for key in result:
 		result[key] /= n_samples
 
@@ -1714,7 +1812,7 @@ def P_instruments_given_signal(horizon_history, n_samples = 500000, min_distance
 	#
 
 	total = sum(sorted(result.values()))
-	assert abs(total - 1.) < 1e-13
+	assert abs(total - 1.) < 1e-13, "result insufficiently well normalized: %s, sum = %g" % (result, total)
 	for key in result:
 		result[key] /= total
 
@@ -1997,6 +2095,119 @@ WHERE
 		snglcoinc.CoincParamsDistributions.addbinnedarrays(self.zero_lag_likelihood_rates, other.zero_lag_likelihood_rates, self.zero_lag_likelihood_pdfs, other.zero_lag_likelihood_pdfs)
 		return self
 
+	@classmethod
+	def new_with_extinction(cls, src):
+		self = cls(None, ())
+		for key, binnedarray in src.background_likelihood_rates.items():
+			self.background_likelihood_rates[key] = binnedarray.copy()
+		for key, binnedarray in src.signal_likelihood_rates.items():
+			self.signal_likelihood_rates[key] = binnedarray.copy()
+		for key, binnedarray in src.zero_lag_likelihood_rates.items():
+			self.zero_lag_likelihood_rates[key] = binnedarray.copy()
+
+		self.finish()
+
+		# Compute the combined rates, we need them for the extinction model
+		self._compute_combined_rates()
+
+		# FIXME We extinct each instrument combination separately,
+		# HOWEVER, only the 'None' key is used, which is all
+		# instruments togther. It is an open question of whether or not
+		# looking at the extinction model separately is a useful
+		# diagnostic tool.  This could change in the future once we
+		# have 3 instruments again to correct each by-instrument PDF
+		# with the survival function
+		for key in self.background_likelihood_rates:
+
+			# pull out both the bg counts and the pdfs, we need 'em both
+			bgcounts_ba = self.background_likelihood_rates[key]
+			bgpdf_ba = self.background_likelihood_pdfs[key]
+			# we also need the zero lag counts to build the extinction model
+			zlagcounts_ba = self.zero_lag_likelihood_rates[key]
+
+			# Only model background above a ln(LR) of 3 or at the 10000 event count whatever is greater
+			likethresh = numpy.searchsorted(bgcounts_ba.bins.upper()[0], max(3, bgcounts_ba.bins.upper()[0][::-1][numpy.searchsorted(zlagcounts_ba.array[::-1].cumsum(), 10000)]))
+			bgcounts_ba.array[:likethresh] = 0.
+			bgpdf_ba.array[:likethresh] = 0.
+			zlagcounts_ba.array[:likethresh] = 0.
+
+			# safety checks
+			assert not numpy.isnan(bgcounts_ba.array).any(), "log likelihood ratio rates contains NaNs"
+			assert not (bgcounts_ba.array < 0.0).any(), "log likelihood ratio rate contains negative values"
+			assert not numpy.isnan(bgpdf_ba.array).any(), "log likelihood ratio pdf contains NaNs"
+			assert not (bgpdf_ba.array < 0.0).any(), "log likelihood ratio pdf contains negative values"
+
+			# grab bins that are not infinite in size
+			finite_bins = numpy.isfinite(bgcounts_ba.bins.volumes())
+			ranks = bgcounts_ba.bins.upper()[0].compress(finite_bins)
+			drank = bgcounts_ba.bins.volumes().compress(finite_bins)
+
+			# figure out the minimum rank
+			fit_min_rank = ranks[likethresh]
+
+			# whittle down the arrays of counts and pdfs
+			bgcounts_ba_array = bgcounts_ba.array.compress(finite_bins)
+			bgpdf_ba_array = bgpdf_ba.array.compress(finite_bins)
+			zlagcounts_ba_array = zlagcounts_ba.array.compress(finite_bins)
+
+			# Issue a warning if we have less than 100,000 events
+			if zlagcounts_ba_array.sum() < 100000:
+				warnings.warn("There are less than 100000 coincidences, extinction effects on background may not be accurately calculated.")
+
+			def extinct(bgcounts_ba_array, bgpdf_ba_array, zlagcounts_ba_array, ranks, drank, fit_min_rank):
+				# Generate arrays of complementary cumulative counts
+				# for background events (monte carlo, pre clustering)
+				# and zero lag events (observed, post clustering)
+				zero_lag_compcumcount = zlagcounts_ba_array[::-1].cumsum()[::-1]
+				bg_compcumcount = bgcounts_ba_array[::-1].cumsum()[::-1]
+
+				# Fit for the number of preclustered, independent coincs by
+				# only considering the observed counts safely in the bulk of
+				# the distribution.  Only do the fit above 10 counts if there
+				# are enough events.
+				rank_range = numpy.logical_and(zero_lag_compcumcount <= max(zero_lag_compcumcount), zero_lag_compcumcount >= min(10, 0.001 * max(zero_lag_compcumcount)))
+
+				# Use curve fit to find the predicted total preclustering
+				# count. First we need an interpolator of the counts
+				obs_counts = interpolate.interp1d(ranks, bg_compcumcount)
+				bg_pdf_interp = interpolate.interp1d(ranks, bgpdf_ba_array)
+
+				def extincted_counts(x, N_ratio, num_clustered = max(zero_lag_compcumcount), norm = obs_counts(fit_min_rank)):
+					out = num_clustered * (1. - numpy.exp(-obs_counts(x) * N_ratio))
+					# This normalization ensures that the left edge does go to num_clustered
+					if (1. - numpy.exp(-norm * N_ratio)) > 0:
+						out /= (1. - numpy.exp(-norm * N_ratio))
+					out[~numpy.isfinite(out)] = 0.
+					return out
+
+				def extincted_pdf(x, N_ratio):
+					out = numpy.exp(numpy.log(N_ratio) - obs_counts(x) * N_ratio + numpy.log(bg_pdf_interp(x)))
+					out[~numpy.isfinite(out)] = 0.
+					return out
+
+				# Fit for the ratio of unclustered to clustered triggers.
+				# Only fit N_ratio over the range of ranks decided above
+				precluster_normalization, precluster_covariance_matrix = curve_fit(
+					extincted_counts,
+					ranks[rank_range],
+					zero_lag_compcumcount.compress(rank_range),
+					sigma = zero_lag_compcumcount.compress(rank_range)**.5,
+					p0 = [1e-4, max(zero_lag_compcumcount)]
+				)
+
+				N_ratio, total_num = precluster_normalization
+
+				return extincted_pdf(ranks, N_ratio)
+
+			# get the extincted background PDF
+			self.background_likelihood_rates[key].array[finite_bins] = extinct(bgcounts_ba_array, bgpdf_ba_array, zlagcounts_ba_array, ranks, drank, fit_min_rank) * drank
+			self.zero_lag_likelihood_rates[key].array[:] = zlagcounts_ba.array[:]
+		# Make sure the PDFs are all updated
+		self.finish()
+
+		# FIXME implement survival function
+		return self, None
+
 	@classmethod
 	def from_xml(cls, xml, name):
 		# find the root of the XML tree containing the
@@ -2057,60 +2268,44 @@ class FAPFAR(object):
 		# none is OK, but then can only compute FAPs, not FARs
 		self.livetime = livetime
 
-		# pull out both the bg counts and the pdfs, we need 'em both
-		bgcounts_ba = ranking_stats.background_likelihood_rates[None]
-		bgpdf_ba = ranking_stats.background_likelihood_pdfs[None]
-		# we also need the zero lag counts to build the extinction model
-		zlagcounts_ba = ranking_stats.zero_lag_likelihood_rates[None]
+		# record trials factor, with safety checks
+		counts = ranking_stats.zero_lag_likelihood_rates[None].array
+		assert not numpy.isnan(counts).any(), "zero lag log likelihood ratio rates contain NaNs"
+		assert (counts >= 0.).all(), "zero lag log likelihood ratio rates contain negative values"
+		self.zero_lag_total_count = counts.sum()
 
-		# safety checks
-		assert not numpy.isnan(bgcounts_ba.array).any(), "log likelihood ratio rates contains NaNs"
-		assert not (bgcounts_ba.array < 0.0).any(), "log likelihood ratio rate contains negative values"
-		assert not numpy.isnan(bgpdf_ba.array).any(), "log likelihood ratio pdf contains NaNs"
-		assert not (bgpdf_ba.array < 0.0).any(), "log likelihood ratio pdf contains negative values"
+		# get noise model ranking stat values and event counts from
+		# bins
+		ranks = ranking_stats.background_likelihood_rates[None].bins[0].upper()
+		counts = ranking_stats.background_likelihood_rates[None].array
+		assert not numpy.isnan(counts).any(), "background log likelihood ratio rates contain NaNs"
+		assert (counts >= 0.).all(), "background log likelihood ratio rates contain negative values"
 
-		# grab bins that are not infinite in size
-		finite_bins = numpy.isfinite(bgcounts_ba.bins.volumes())
-		ranks = bgcounts_ba.bins.upper()[0].compress(finite_bins)
-		drank = bgcounts_ba.bins.volumes().compress(finite_bins)
-
-		# whittle down the arrays of counts and pdfs
-		bgcounts_ba_array = bgcounts_ba.array.compress(finite_bins)
-		bgpdf_ba_array = bgpdf_ba.array.compress(finite_bins)
-		zlagcounts_ba_array = zlagcounts_ba.array.compress(finite_bins)
-
-		# get the extincted background PDF
-		self.zero_lag_total_count = zlagcounts_ba_array.sum()
-		extinct_bf_pdf = self.extinct(bgcounts_ba_array, bgpdf_ba_array, zlagcounts_ba_array, ranks)
-
-		# cumulative distribution function and its complement.
-		# it's numerically better to recompute the ccdf by
-		# reversing the array of weights than trying to subtract
-		# the cdf from 1.
-		weights = extinct_bf_pdf * drank
-		cdf = weights.cumsum()
-		cdf /= cdf[-1]
-		ccdf = weights[::-1].cumsum()[::-1]
+		# complementary cumulative distribution function
+		ccdf = counts[::-1].cumsum()[::-1]
 		ccdf /= ccdf[0]
 
-		# cdf boundary condition:  cdf = 1/e at the ranking
-		# statistic threshold so that self.far_from_rank(threshold)
-		# * livetime = observed count of events above threshold.
-		# FIXME this doesn't actually work.
-		# FIXME not doing it doesn't actually work.
-		# ccdf *= 1. - 1. / math.e
-		# cdf *= 1. - 1. / math.e
-		# cdf += 1. / math.e
+		# ccdf is P(ranking stat > threshold | a candidate).  we
+		# need P(ranking stat > threshold), i.e. need to correct
+		# for the possibility that no candidate is present.
+		# specifically, the ccdf needs to =1-1/e at the candidate
+		# identification threshold, and cdf=1/e at the candidate
+		# threshold, in order for FAR(threshold) * livetime to
+		# equal the actual observed number of candidates.
+		ccdf = poisson_p_not_0(ccdf)
 
-		# last checks that the CDF and CCDF are OK
-		assert not numpy.isnan(cdf).any(), "log likelihood ratio CDF contains NaNs"
+		# interpolator won't accept infinite co-ordinates so need
+		# to remove the last bin
+		ranks = ranks[:-1]
+		ccdf = ccdf[:-1]
+
+		# safety checks
+		assert not numpy.isnan(ranks).any(), "log likelihood ratio co-ordinates contain NaNs"
+		assert not numpy.isinf(ranks).any(), "log likelihood ratio co-ordinates are not all finite"
 		assert not numpy.isnan(ccdf).any(), "log likelihood ratio CCDF contains NaNs"
-		assert ((0. <= cdf) & (cdf <= 1.)).all(), "log likelihood ratio CDF failed to be normalized"
 		assert ((0. <= ccdf) & (ccdf <= 1.)).all(), "log likelihood ratio CCDF failed to be normalized"
-		assert (abs(1. - (cdf[:-1] + ccdf[1:])) < 1e-12).all(), "log likelihood ratio CDF + CCDF != 1 (max error = %g)" % abs(1. - (cdf[:-1] + ccdf[1:])).max()
 
-		# build interpolators
-		self.cdf_interpolator = interpolate.interp1d(ranks, cdf)
+		# build interpolator.
 		self.ccdf_interpolator = interpolate.interp1d(ranks, ccdf)
 
 		# record min and max ranks so we know which end of the ccdf
@@ -2118,87 +2313,34 @@ class FAPFAR(object):
 		self.minrank = min(ranks)
 		self.maxrank = max(ranks)
 
-	def extinct(self, bgcounts_ba_array, bgpdf_ba_array, zlagcounts_ba_array, ranks):
-		# Generate arrays of complementary cumulative counts
-		# for background events (monte carlo, pre clustering)
-		# and zero lag events (observed, post clustering)
-		zero_lag_compcumcount = zlagcounts_ba_array[::-1].cumsum()[::-1]
-		bg_compcumcount = bgcounts_ba_array[::-1].cumsum()[::-1]
-
-		# Fit for the number of preclustered, independent coincs by
-		# only considering the observed counts safely in the bulk of
-		# the distribution.  Only do the fit above 10 counts and below
-		# 10000, unless that can't be met and trigger a warning
-		fit_min_rank = 1.
-		fit_min_counts = min(10., self.zero_lag_total_count / 10. + 1)
-		fit_max_counts = min(10000., self.zero_lag_total_count / 10. + 2) # the +2 gaurantees that fit_max_counts > fit_min_counts
-		rank_range = numpy.logical_and(ranks > fit_min_rank, numpy.logical_and(zero_lag_compcumcount < fit_max_counts, zero_lag_compcumcount > fit_min_counts))
-		if fit_max_counts < 10000.:
-			warnings.warn("There are less than 100000 coincidences, extinction effects on background may not be accurately calculated, which will decrease the accuracy of the combined instruments background estimation.")
-		if zero_lag_compcumcount.compress(rank_range).size < 1:
-			raise ValueError("not enough zero lag data to fit background")
-
-		# Use curve fit to find the predicted total preclustering
-		# count. First we need an interpolator of the counts
-		obs_counts = interpolate.interp1d(ranks, bg_compcumcount)
-		bg_pdf_interp = interpolate.interp1d(ranks, bgpdf_ba_array)
-
-		def extincted_counts(x, N_ratio):
-			out = max(zero_lag_compcumcount) * (1. - numpy.exp(-obs_counts(x) * N_ratio))
-			out[~numpy.isfinite(out)] = 0.
-			return out
-
-		def extincted_pdf(x, N_ratio):
-			out = numpy.exp(numpy.log(N_ratio) - obs_counts(x) * N_ratio + numpy.log(bg_pdf_interp(x)))
-			out[~numpy.isfinite(out)] = 0.
-			return out
-
-		# Fit for the ratio of unclustered to clustered triggers.
-		# Only fit N_ratio over the range of ranks decided above
-		precluster_normalization, precluster_covariance_matrix = curve_fit(
-			extincted_counts,
-			ranks[rank_range],
-			zero_lag_compcumcount.compress(rank_range),
-			sigma = zero_lag_compcumcount.compress(rank_range)**.5,
-			p0 = 1e-4
-		)
-
-		N_ratio = precluster_normalization[0]
-
-		return extincted_pdf(ranks, N_ratio)
+	@assert_probability
+	def ccdf_from_rank(self, rank):
+		rank = max(self.minrank, min(self.maxrank, rank))
+		return float(self.ccdf_interpolator(rank))
 
+	@assert_probability
 	def fap_from_rank(self, rank):
 		# implements equation (8) from Phys. Rev. D 88, 024025.
 		# arXiv:1209.0718
-		rank = max(self.minrank, min(self.maxrank, rank))
-		fap = float(self.ccdf_interpolator(rank))
-		return fap_after_trials(fap, self.zero_lag_total_count)
+		return fap_after_trials(self.ccdf_from_rank(rank), self.zero_lag_total_count)
 
 	def far_from_rank(self, rank):
 		# implements equation (B4) of Phys. Rev. D 88, 024025.
 		# arXiv:1209.0718.  the return value is divided by T to
 		# convert events/experiment to events/second.
 		assert self.livetime is not None, "cannot compute FAR without livetime"
-		rank = max(self.minrank, min(self.maxrank, rank))
 		# true-dismissal probability = 1 - single-event false-alarm
 		# probability, the integral in equation (B4)
-		tdp = float(self.cdf_interpolator(rank))
-		try:
-			log_tdp = math.log(tdp)
-		except ValueError:
-			# TDP = 0 --> FAR = +inf
-			return PosInf
-		if log_tdp >= -1e-9:
-			# rare event:  avoid underflow by using log1p(-FAP)
-			log_tdp = math.log1p(-float(self.ccdf_interpolator(rank)))
+		log_tdp = math.log1p(-self.ccdf_from_rank(rank))
 		return self.zero_lag_total_count * -log_tdp / self.livetime
 
-	def assign_faps(self, connection):
-		# assign false-alarm probabilities
-		# FIXME:  choose a function name more likely to be unique?
-		# FIXME:  abusing false_alarm_rate column, move for a
-		# false_alarm_probability column??
+	def assign_fapfars(self, connection):
+		# assign false-alarm probabilities and false-alarm rates
+		# FIXME:  choose function names more likely to be unique?
+		# FIXME:  abusing false_alarm_rate column to store FAP,
+		# move to a false_alarm_probability column??
 		connection.create_function("fap", 1, self.fap_from_rank)
+		connection.create_function("far", 1, self.far_from_rank)
 		connection.cursor().execute("""
 UPDATE
 	coinc_inspiral
@@ -2210,17 +2352,7 @@ SET
 			coinc_event
 		WHERE
 			coinc_event.coinc_event_id == coinc_inspiral.coinc_event_id
-	)
-""")
-
-	def assign_fars(self, connection):
-		# assign false-alarm rates
-		# FIXME:  choose a function name more likely to be unique?
-		connection.create_function("far", 1, self.far_from_rank)
-		connection.cursor().execute("""
-UPDATE
-	coinc_inspiral
-SET
+	),
 	combined_far = (
 		SELECT
 			far(coinc_event.likelihood)
