@@ -686,6 +686,7 @@ class Data(object):
 					self.ranking_data, seglists = far.parse_likelihood_control_doc(ligolw_utils.load_filename(self.marginalized_likelihood_file, verbose = self.verbose, contenthandler = far.ThincaCoincParamsDistributions.LIGOLWContentHandler))[1:]
 					if self.ranking_data is None:
 						raise ValueError("\"%s\" does not contain ranking statistic PDFs" % self.marginalized_likelihood_file)
+					self.ranking_data = self.ranking_data.new_with_extinction(self.ranking_data)[0]
 					self.ranking_data.finish(verbose = self.verbose)
 					self.fapfar = far.FAPFAR(self.ranking_data, livetime = far.get_live_time(seglists))
 
@@ -891,22 +892,6 @@ class Data(object):
 				fobj = StringIO.StringIO()
 				ligolw_utils.write_fileobj(self.__get_likelihood_file(), fobj, gz = True, trap_signals = None)
 				common_messages.append(("ranking statistic PDFs", "ranking_data.xml.gz", "ranking statistic", fobj.getvalue()))
-
-				# FIXME:  stop doing this when
-				# .new_with_extinction() patch is applied.
-				# right now the ranking statistic PDF and
-				# CCDF uploaded to gracedb is missing the
-				# extinction model and and the zero-lag
-				# histogram is required to construct it.
-				# after .new_with_extinction() what gest
-				# uploaded will have had the extinction
-				# model applied and this auxiliary
-				# information will no longer be required
-				if self.verbose:
-					print >>sys.stderr, "generating zero_lag_ranking_stats.xml.gz ..."
-				fobj = StringIO.StringIO()
-				ligolw_utils.write_fileobj(self.__get_zero_lag_ranking_stats_file(), fobj, gz = True, trap_signals = None)
-				common_messages.append(("zero-lag ranking statistic histogram", "zero_lag_ranking_stats.xml.gz", "ranking statistic", fobj.getvalue()))
 				del fobj
 
 			#
